@@ -1,4 +1,4 @@
-/* $Id: browser.c 4173 2007-10-11 05:01:32Z dolorous $ */
+/* $Id: browser.c 4226 2008-03-14 04:08:51Z astyanax $ */
 /**************************************************************************
  *   browser.c                                                            *
  *                                                                        *
@@ -60,13 +60,15 @@ char *do_browser(char *path, DIR *dir)
 	/* The last answer the user typed at the statusbar prompt. */
     size_t old_selected;
 	/* The selected file we had before the current selected file. */
+    const sc *s;
+    const subnfunc *f;
 
     curs_set(0);
     blank_statusbar();
 #if !defined(DISABLE_HELP) || !defined(DISABLE_MOUSE)
-    currshortcut = browser_list;
+    currmenu = MBROWSER;
 #endif
-    bottombars(browser_list);
+    bottombars(MBROWSER);
     wnoutrefresh(bottomwin);
 
     UNSET(CONST_UPDATE);
@@ -123,12 +125,10 @@ char *do_browser(char *path, DIR *dir)
 	old_selected = selected;
 
 	kbinput = get_kbinput(edit, &meta_key, &func_key);
-	parse_browser_input(&kbinput, &meta_key, &func_key);
 
-	switch (kbinput) {
 #ifndef DISABLE_MOUSE
-	    case KEY_MOUSE:
-		{
+        if (kbinput == KEY_MOUSE) {
+
 		    int mouse_x, mouse_y;
 
 		    /* We can click on the edit window to select a
@@ -156,74 +156,74 @@ char *do_browser(char *path, DIR *dir)
 			 * time, put back the Enter key so that it's
 			 * read in. */
 			if (old_selected == selected)
-			    unget_kbinput(NANO_ENTER_KEY, FALSE, FALSE);
+			    unget_kbinput(sc_seq_or(do_enter, 0), FALSE, FALSE);
 		    }
-		}
-		break;
+	}
 #endif /* !DISABLE_MOUSE */
-	    /* Redraw the screen. */
-	    case NANO_REFRESH_KEY:
+
+	parse_browser_input(&kbinput, &meta_key, &func_key);
+        s = get_shortcut(MBROWSER, &kbinput, &meta_key, &func_key);
+        if (!s)
+            continue;
+        f = sctofunc((sc *) s);
+        if (!f)
+            break;
+
+	if (f->scfunc == total_refresh) {
 		total_redraw();
-		break;
-	    case NANO_HELP_KEY:
+	} else if (f->scfunc == do_help_void) {
 #ifndef DISABLE_HELP
 		do_browser_help();
 		curs_set(0);
 #else
 		nano_disabled_msg();
 #endif
-		break;
 	    /* Search for a filename. */
-	    case NANO_WHEREIS_KEY:
+	} else if (f->scfunc == do_search) {
 		curs_set(1);
 		do_filesearch();
 		curs_set(0);
-		break;
 	    /* Search for another filename. */
-	    case NANO_WHEREIS_NEXT_KEY:
+	} else if (f->scfunc == (void *) whereis_next_msg) {
 		do_fileresearch();
-		break;
-	    case NANO_PREVPAGE_KEY:
+	} else if (f->scfunc == do_page_up) {
 		if (selected >= (editwinrows + fileline % editwinrows) *
 			width)
 		    selected -= (editwinrows + fileline % editwinrows) *
 			width;
 		else
 		    selected = 0;
-		break;
-	    case NANO_NEXTPAGE_KEY:
+	} else if (f->scfunc == do_page_down) {
 		selected += (editwinrows - fileline % editwinrows) *
 			width;
 		if (selected > filelist_len - 1)
 		    selected = filelist_len - 1;
-		break;
-	    case NANO_FIRSTFILE_METAKEY:
+	} else if (f->scfunc == (void *) first_file_msg) {
 		if (meta_key)
 		    selected = 0;
-		break;
-	    case NANO_LASTFILE_METAKEY:
+	} else if (f->scfunc == (void *) last_file_msg) {
 		if (meta_key)
 		    selected = filelist_len - 1;
-		break;
 	    /* Go to a specific directory. */
-	    case NANO_GOTODIR_KEY:
+	} else if (f->scfunc == (void *) goto_dir_msg) {
 		curs_set(1);
 
 		i = do_prompt(TRUE,
 #ifndef DISABLE_TABCOMP
 			FALSE,
 #endif
-			gotodir_list, ans,
+			MGOTODIR, ans,
+			&meta_key, &func_key,
 #ifndef NANO_TINY
 			NULL,
 #endif
-			browser_refresh, _("Go To Directory"));
+			browser_refresh, N_("Go To Directory"));
 
 		curs_set(0);
 #if !defined(DISABLE_HELP) || !defined(DISABLE_MOUSE)
-		currshortcut = browser_list;
+		currmenu = MBROWSER;
 #endif
-		bottombars(browser_list);
+		bottombars(MBROWSER);
 
 		/* If the directory begins with a newline (i.e. an
 		 * encoded null), treat it as though it's blank. */
@@ -238,7 +238,7 @@ char *do_browser(char *path, DIR *dir)
 		     * answer in ans, so that the file list is displayed
 		     * again, the prompt is displayed again, and what we
 		     * typed before at the prompt is displayed again. */
-		    unget_kbinput(NANO_GOTOLINE_KEY, FALSE, FALSE);
+		    unget_kbinput(sc_seq_or(do_gotolinecolumn_void, 0), FALSE, FALSE);
 		    ans = mallocstrcpy(ans, answer);
 		    break;
 		}
@@ -285,23 +285,19 @@ char *do_browser(char *path, DIR *dir)
 		free(path);
 		path = new_path;
 		goto change_browser_directory;
-	    case NANO_PREVLINE_KEY:
+	} else if (f->scfunc == do_up_void) {
 		if (selected >= width)
 		    selected -= width;
-		break;
-	    case NANO_BACK_KEY:
+	} else if (f->scfunc == do_left) {
 		if (selected > 0)
 		    selected--;
-		break;
-	    case NANO_NEXTLINE_KEY:
+	} else if (f->scfunc == do_down_void) {
 		if (selected + width <= filelist_len - 1)
 		    selected += width;
-		break;
-	    case NANO_FORWARD_KEY:
+	} else if (f->scfunc == do_right) {
 		if (selected < filelist_len - 1)
 		    selected++;
-		break;
-	    case NANO_ENTER_KEY:
+	} else if (f->scfunc == do_enter) {
 		/* We can't move up from "/". */
 		if (strcmp(filelist[selected], "/..") == 0) {
 		    statusbar(_("Can't move up a directory"));
@@ -359,12 +355,10 @@ char *do_browser(char *path, DIR *dir)
 		/* Start over again with the new path value. */
 		goto change_browser_directory;
 	    /* Abort the file browser. */
-	    case NANO_EXIT_KEY:
+	} else if (f->scfunc == do_exit) {
 		abort = TRUE;
-		break;
 	}
     }
-
     titlebar(NULL);
     edit_refresh();
     curs_set(1);
@@ -552,37 +546,36 @@ void browser_init(const char *path, DIR *dir)
  * shortcut keys. */
 void parse_browser_input(int *kbinput, bool *meta_key, bool *func_key)
 {
-    get_shortcut(browser_list, kbinput, meta_key, func_key);
+    get_shortcut(MBROWSER, kbinput, meta_key, func_key);
 
     /* Pico compatibility. */
     if (!*meta_key) {
 	switch (*kbinput) {
 	    case ' ':
-		*kbinput = NANO_NEXTPAGE_KEY;
+		*kbinput = sc_seq_or(do_page_down, 0);
 		break;
 	    case '-':
-		*kbinput = NANO_PREVPAGE_KEY;
+		*kbinput = sc_seq_or(do_page_up, 0);
 		break;
 	    case '?':
-		*kbinput = NANO_HELP_KEY;
+		*kbinput = sc_seq_or(do_help_void, 0);
 		break;
-	    /* Cancel is equivalent to Exit here. */
-	    case NANO_CANCEL_KEY:
+	    /* Cancel equivalent to Exit here. */
 	    case 'E':
 	    case 'e':
-		*kbinput = NANO_EXIT_KEY;
+		*kbinput = sc_seq_or(do_exit, 0);
 		break;
 	    case 'G':
 	    case 'g':
-		*kbinput = NANO_GOTODIR_KEY;
+		*kbinput = sc_seq_or((void *) goto_dir_msg, 0);
 		break;
 	    case 'S':
 	    case 's':
-		*kbinput = NANO_ENTER_KEY;
+		*kbinput = sc_seq_or(do_enter, 0);
 		break;
 	    case 'W':
 	    case 'w':
-		*kbinput = NANO_WHEREIS_KEY;
+		*kbinput = sc_seq_or(do_search, 0);
 		break;
 	}
     }
@@ -765,6 +758,8 @@ int filesearch_init(void)
 {
     int i = 0;
     char *buf;
+    bool meta_key, func_key;
+    const sc *s;
     static char *backupstring = NULL;
 	/* The search string we'll be using. */
 
@@ -796,7 +791,8 @@ int filesearch_init(void)
 #ifndef DISABLE_TABCOMP
 	TRUE,
 #endif
-	whereis_file_list, backupstring,
+	MWHEREISFILE, backupstring,
+	&meta_key, &func_key, 
 #ifndef NANO_TINY
 	&search_history,
 #endif
@@ -832,9 +828,8 @@ int filesearch_init(void)
 	statusbar(_("Cancelled"));
 	return -1;
     } else {
-	switch (i) {
-	    case -2:		/* It's an empty string. */
-	    case 0:		/* It's a new string. */
+        s = get_shortcut(MBROWSER, &i, &meta_key, &func_key);
+	if (i == -2 || i == 0) {
 #ifdef HAVE_REGEX_H
 		/* Use last_search if answer is an empty string, or
 		 * answer if it isn't. */
@@ -842,26 +837,26 @@ int filesearch_init(void)
 			last_search : answer))
 		    return -1;
 #endif
-		break;
+	} else
 #ifndef NANO_TINY
-	    case TOGGLE_CASE_KEY:
+	if (s && s->scfunc == (void *) case_sens_msg) {
 		TOGGLE(CASE_SENSITIVE);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
-	    case TOGGLE_BACKWARDS_KEY:
+	} else if (s && s->scfunc == (void *) backwards_msg) {
 		TOGGLE(BACKWARDS_SEARCH);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
+	} else
 #endif
 #ifdef HAVE_REGEX_H
-	    case NANO_REGEXP_KEY:
+	if (s && s->scfunc == (void *) regexp_msg) {
 		TOGGLE(USE_REGEXP);
 		backupstring = mallocstrcpy(backupstring, answer);
 		return 1;
+	} else
 #endif
-	    default:
-		return -1;
-	}
+	    return -1;
     }
 
     return 0;
@@ -955,8 +950,8 @@ void findnextfile_wrap_reset(void)
  * search, if any. */
 void filesearch_abort(void)
 {
-    currshortcut = browser_list;
-    bottombars(browser_list);
+    currmenu = MBROWSER;
+    bottombars(MBROWSER);
 #ifdef HAVE_REGEX_H
     regexp_cleanup();
 #endif
