@@ -1,12 +1,12 @@
-/* $Id: cut.c,v 1.61 2007/01/01 05:15:32 dolorous Exp $ */
+/* $Id: cut.c 4291 2008-08-03 04:48:05Z astyanax $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
- *   Copyright (C) 2005, 2006, 2007 David Lawrence Ramsey                 *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -28,7 +28,6 @@
 
 static bool keep_cutbuffer = FALSE;
 	/* Should we keep the contents of the cutbuffer? */
-static filestruct *cutbottom = NULL;
 	/* Pointer to the end of the cutbuffer. */
 
 /* Indicate that we should no longer keep the contents of the
@@ -114,7 +113,7 @@ void cut_to_eof(void)
  * position to the end of the file into the cutbuffer. */
 void do_cut_text(
 #ifndef NANO_TINY
-	bool copy_text, bool cut_till_end
+	bool copy_text, bool cut_till_end, bool undoing
 #else
 	void
 #endif
@@ -163,6 +162,7 @@ void do_cut_text(
     keep_cutbuffer = TRUE;
 
 #ifndef NANO_TINY
+
     if (cut_till_end) {
 	/* If cut_till_end is TRUE, move all text up to the end of the
 	 * file into the cutbuffer. */
@@ -204,12 +204,14 @@ void do_cut_text(
 	 * disturbing the text. */
 	if (!old_no_newlines)
 	    UNSET(NO_NEWLINES);
-    } else
+    } else if (!undoing)
+	update_undo(CUT);
 #endif
 	/* Leave the text in the cutbuffer, and mark the file as
 	 * modified. */
 	set_modified();
 
+
     /* Update the screen. */
     edit_refresh();
 
@@ -221,9 +223,12 @@ void do_cut_text(
 /* Move text from the current filestruct into the cutbuffer. */
 void do_cut_text_void(void)
 {
+#ifndef NANO_TINY
+    add_undo(CUT);
+#endif
     do_cut_text(
 #ifndef NANO_TINY
-	FALSE, FALSE
+	FALSE, FALSE, FALSE
 #endif
 	);
 }
@@ -233,13 +238,16 @@ void do_cut_text_void(void)
  * back into the filestruct afterward. */
 void do_copy_text(void)
 {
-    do_cut_text(TRUE, FALSE);
+    do_cut_text(TRUE, FALSE, FALSE);
 }
 
 /* Cut from the current cursor position to the end of the file. */
 void do_cut_till_end(void)
 {
-    do_cut_text(FALSE, TRUE);
+#ifndef NANO_TINY
+    add_undo(CUTTOEND);
+#endif
+    do_cut_text(FALSE, TRUE, FALSE);
 }
 #endif /* !NANO_TINY */
 
