@@ -1,4 +1,4 @@
-/* $Id: files.c 4260 2008-05-31 23:09:40Z astyanax $ */
+/* $Id: files.c 4299 2008-08-09 09:31:33Z astyanax $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -75,6 +75,8 @@ void initialize_buffer(void)
     openfile->fmt = NIX_FILE;
 
     openfile->current_stat = NULL;
+    openfile->undotop = NULL;
+    openfile->current_undo = NULL;
 #endif
 #ifdef ENABLE_COLOR
     openfile->colorstrings = NULL;
@@ -99,7 +101,7 @@ void initialize_buffer_text(void)
 
 /* If it's not "", filename is a file to open.  We make a new buffer, if
  * necessary, and then open and read the file, if applicable. */
-void open_buffer(const char *filename)
+void open_buffer(const char *filename, bool undoable)
 {
     bool new_buffer = (openfile == NULL
 #ifdef ENABLE_MULTIBUFFER
@@ -140,7 +142,7 @@ void open_buffer(const char *filename)
     /* If we have a non-new file, read it in.  Then, if the buffer has
      * no stat, update the stat, if applicable. */
     if (rc == 0) {
-	read_file(f, filename);
+	read_file(f, filename, undoable);
 #ifndef NANO_TINY
 	if (openfile->current_stat == NULL) {
 	    openfile->current_stat =
@@ -190,7 +192,7 @@ void replace_buffer(const char *filename)
 
     /* If we have a non-new file, read it in. */
     if (rc == 0)
-	read_file(f, filename);
+	read_file(f, filename, FALSE);
 
     /* Move back to the beginning of the first line of the buffer. */
     openfile->current = openfile->fileage;
@@ -337,8 +339,9 @@ filestruct *read_line(char *buf, filestruct *prevnode, bool
 }
 
 /* Read an open file into the current buffer.  f should be set to the
- * open file, and filename should be set to the name of the file. */
-void read_file(FILE *f, const char *filename)
+ * open file, and filename should be set to the name of the file.
+   undoable  means do we want to create undo records to try and undo this */
+void read_file(FILE *f, const char *filename, bool undoable)
 {
     size_t num_lines = 0;
 	/* The number of lines in the file. */
@@ -369,6 +372,11 @@ void read_file(FILE *f, const char *filename)
     buf = charalloc(bufx);
     buf[0] = '\0';
 
+#ifndef NANO_TINY
+    if (undoable)
+	add_undo(INSERT);
+#endif
+
     if (openfile->current == openfile->fileage)
 	first_line_ins = TRUE;
     else
@@ -487,7 +495,7 @@ void read_file(FILE *f, const char *filename)
     /* If we didn't get a file and we don't already have one, open a
      * blank buffer. */
     if (fileptr == NULL)
-	open_buffer("");
+	open_buffer("", FALSE);
 
     /* Attach the file we got to the filestruct.  If we got a file of
      * zero bytes, don't do anything. */
@@ -559,6 +567,9 @@ void read_file(FILE *f, const char *filename)
     openfile->placewewant = xplustabs();
 
 #ifndef NANO_TINY
+    if (undoable)
+	update_undo(INSERT);
+
     if (format == 3)
 	statusbar(
 		P_("Read %lu line (Converted from DOS and Mac format)",
@@ -851,7 +862,7 @@ void do_insertfile(
 #ifdef ENABLE_MULTIBUFFER
 		if (ISSET(MULTIBUFFER))
 		    /* Open a blank buffer. */
-		    open_buffer("");
+		    open_buffer("", FALSE);
 #endif
 
 		/* Save the command's output in the current buffer. */
@@ -875,7 +886,7 @@ void do_insertfile(
 
 		/* Save the file specified in answer in the current
 		 * buffer. */
-		open_buffer(answer);
+		open_buffer(answer, TRUE);
 #ifndef NANO_TINY
 	    }
 #endif
@@ -1463,14 +1474,15 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 	    free(backuptemp);
 	    backuptemp = get_next_filename(backupname, "~");
 	    if (*backuptemp == '\0') {
-		statusbar(_("Error writing %s: %s"), backupname,
+		statusbar(_("Error writing backup file %s: %s"), backupname,
 		    _("Too many backup files?"));
 		free(backuptemp);
 		free(backupname);
-		/* If we can't write to the backup, go on, since only
-		 * saving the original file is better than saving
-		 * nothing. */
-		goto skip_backup;
+		/* If we can't write to the backup, DONT go on, since
+		   whatever caused the backup file to fail (e.g. disk
+		   full may well cause the real file write to fail, which
+		   means we could lose both the backup and the original! */
+		goto cleanup_and_exit;
 	    } else {
 		free(backupname);
 		backupname = backuptemp;
@@ -1487,14 +1499,16 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 
 	if (backup_file == NULL || chmod(backupname,
 		openfile->current_stat->st_mode) == -1) {
-	    statusbar(_("Error writing %s: %s"), backupname,
+	    statusbar(_("Error writing backup file %s: %s"), backupname,
 		strerror(errno));
 	    free(backupname);
 	    if (backup_file != NULL)
 		fclose(backup_file);
-	    /* If we can't write to the backup, go on, since only saving
-	     * the original file is better than saving nothing. */
-	    goto skip_backup;
+	    /* If we can't write to the backup, DONT go on, since
+	       whatever caused the backup file to fail (e.g. disk
+	       full may well cause the real file write to fail, which
+	       means we could lose both the backup and the original! */
+	    goto cleanup_and_exit;
 	}
 
 #ifdef DEBUG
@@ -1514,17 +1528,19 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 			strerror(errno));
 		beep();
 	    } else
-		statusbar(_("Error writing %s: %s"), backupname,
+		statusbar(_("Error writing backup file %s: %s"), backupname,
 			strerror(errno));
-	    /* If we can't read from or write to the backup, go on,
-	     * since only saving the original file is better than saving
-	     * nothing. */
+	    /* If we can't write to the backup, DONT go on, since
+	       whatever caused the backup file to fail (e.g. disk
+	       full may well cause the real file write to fail, which
+	       means we could lose both the backup and the original! */
+	    goto cleanup_and_exit;
 	}
 
 	free(backupname);
     }
 
-  skip_backup:
+    skip_backup:
 #endif /* !NANO_TINY */
 
     /* If NOFOLLOW_SYMLINKS is set and the file is a link, we aren't
