@@ -1,5 +1,4 @@
-/* $Id: global.c 4264 2008-06-03 08:09:05Z astyanax $ 
-*/
+/* $Id: global.c 4283 2008-07-31 04:24:04Z astyanax $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -22,11 +21,12 @@
  *                                                                        *
  **************************************************************************/
 
+#include "proto.h"
+
 #include <ctype.h>
 #include <string.h>
 #include <strings.h>
 #include "assert.h"
-#include "proto.h"
 
 /* Global variables. */
 #ifndef NANO_TINY
@@ -69,6 +69,7 @@ int editwinrows = 0;
 
 filestruct *cutbuffer = NULL;
 	/* The buffer where we store cut text. */
+filestruct *cutbottom = NULL;
 #ifndef DISABLE_JUSTIFY
 filestruct *jusbuffer = NULL;
 	/* The buffer where we store unjustified text. */
@@ -150,11 +151,6 @@ const shortcut *currshortcut;
 int currmenu;
 	/* The currently loaded menu */
 
-#ifndef NANO_TINY
-toggle *toggles = NULL;
-	/* The global toggle list. */
-#endif
-
 sc *sclist = NULL;
 	/* New shortcut key struct */
 subnfunc *allfuncs = NULL;
@@ -528,6 +524,8 @@ void shortcut_init(bool unjustify)
 	N_("Copy the current line and store it in the cutbuffer");
     const char *nano_indent_msg = N_("Indent the current line");
     const char *nano_unindent_msg = N_("Unindent the current line");
+    const char *nano_undo_msg = N_("Undo the last operation");
+    const char *nano_redo_msg = N_("Redo the last undone operation");
 #endif
     const char *nano_forward_msg = N_("Go forward one character");
     const char *nano_back_msg = N_("Go back one character");
@@ -698,6 +696,7 @@ void shortcut_init(bool unjustify)
     add_to_funcs(do_page_down, MMAIN|MHELP,
 	next_page_msg, IFSCHELP(nano_nextpage_msg), TRUE, VIEW);
 
+
     /* TRANSLATORS: Try to keep this at most 10 characters. */
     add_to_funcs(do_cut_text_void, MMAIN, N_("Cut Text"), IFSCHELP(nano_cut_msg),
 	FALSE, NOVIEW);
@@ -771,7 +770,13 @@ void shortcut_init(bool unjustify)
 	nano_indent_msg, FALSE, NOVIEW);
 
     add_to_funcs(do_unindent, MMAIN, N_("Unindent Text"),
-	nano_unindent_msg, TRUE, NOVIEW);
+	nano_unindent_msg, FALSE, NOVIEW);
+
+    add_to_funcs(do_undo, MMAIN, N_("Undo"),
+	nano_undo_msg, FALSE, NOVIEW);
+
+    add_to_funcs(do_redo, MMAIN, N_("Redo"),
+	nano_redo_msg, TRUE, NOVIEW);
 
 #endif
 
@@ -1055,6 +1060,8 @@ void shortcut_init(bool unjustify)
     add_to_sclist(MMAIN, "M-6", do_copy_text, 0, TRUE);
     add_to_sclist(MMAIN, "M-}", do_indent_void, 0, TRUE);
     add_to_sclist(MMAIN, "M-{", do_unindent, 0, TRUE);
+    add_to_sclist(MMAIN, "M-U", do_undo, 0, TRUE);
+    add_to_sclist(MMAIN, "M-E", do_redo, 0, TRUE);
     add_to_sclist(MALL, "^F", do_right, 0, TRUE);
     add_to_sclist(MALL, "^B", do_left, 0, TRUE);
     add_to_sclist(MMAIN, "^Space", do_next_word_void, 0, TRUE);
@@ -1072,13 +1079,13 @@ void shortcut_init(bool unjustify)
     add_to_sclist(MALL, "khome", do_home, 0, TRUE);
     add_to_sclist(MALL, "^E", do_end, 0, TRUE);
     add_to_sclist(MALL, "kend", do_end, 0, TRUE);
-#ifndef DISABLE_JUSTIFY
 #ifndef NANO_TINY
     add_to_sclist(MWHEREIS|MREPLACE|MREPLACE2|MWHEREISFILE, "^P", (void *) prev_history_msg, 0, FALSE);
     add_to_sclist(MWHEREIS|MREPLACE|MREPLACE2|MWHEREISFILE, "kup", (void *) prev_history_msg, 0, FALSE);
     add_to_sclist(MWHEREIS|MREPLACE|MREPLACE2|MWHEREISFILE, "^N", (void *) next_history_msg, 0, FALSE);
     add_to_sclist(MWHEREIS|MREPLACE|MREPLACE2|MWHEREISFILE, "kdown", (void *) next_history_msg, 0, FALSE);
 #endif
+#ifndef DISABLE_JUSTIFY
     add_to_sclist(MWHEREIS|MREPLACE|MREPLACE2,
 	"^W", do_para_begin_void, 0, TRUE);
     add_to_sclist(MWHEREIS|MREPLACE|MREPLACE2,
@@ -1333,6 +1340,10 @@ sc *strtosc(int menu, char *input)
 	s->scfunc = do_find_bracket;
     else if (!strcasecmp(input, "wordcount"))
 	s->scfunc = do_wordlinechar_count;
+    else if (!strcasecmp(input, "undo"))
+	s->scfunc = do_undo;
+    else if (!strcasecmp(input, "redo"))
+	s->scfunc = do_redo;
     else if (!strcasecmp(input, "prevhistory")) {
 	s->scfunc = (void *) prev_history_msg;
 	s->execute = FALSE;
@@ -1436,6 +1447,8 @@ sc *strtosc(int menu, char *input)
 	s->scfunc = do_enter;
     else if (!strcasecmp(input, "delete"))
 	s->scfunc = do_delete;
+    else if (!strcasecmp(input, "backspace"))
+	s->scfunc = do_backspace;
     else if (!strcasecmp(input, "refresh"))
 	s->scfunc = total_refresh;
     else if (!strcasecmp(input, "casesens")) {
@@ -1576,15 +1589,6 @@ void thanks_for_all_the_fish(void)
 #ifndef DISABLE_JUSTIFY
     if (jusbuffer != NULL)
 	free_filestruct(jusbuffer);
-#endif
-#ifndef NANO_TINY
-    /* Free the memory associated with each toggle. */
-    while (toggles != NULL) {
-	toggle *t = toggles;
-
-	toggles = toggles->next;
-	free(t);
-    }
 #endif
     /* Free the memory associated with each open file buffer. */
     if (openfile != NULL)
