@@ -1,4 +1,4 @@
-/* $Id: help.c 4233 2008-03-16 23:57:14Z astyanax $ */
+/* $Id: help.c 4247 2008-03-20 05:56:46Z astyanax $ */
 /**************************************************************************
  *   help.c                                                               *
  *                                                                        *
@@ -414,7 +414,7 @@ void help_init(void)
 
 	for (s = sclist; s != NULL; s = s->next)
             if (s->scfunc == (void *) do_toggle)
-		allocsize += strlen(flagtostr(s->toggle)) + endis_len + 9;
+		allocsize += strlen(_(flagtostr(s->toggle))) + endis_len + 9;
 
     }
 #endif
