@@ -1,4 +1,4 @@
-/* $Id: nano.c 4256 2008-05-31 22:23:16Z astyanax $ */
+/* $Id: nano.c 4291 2008-08-03 04:48:05Z astyanax $ */ 
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -32,7 +32,9 @@
 #include <errno.h>
 #include <ctype.h>
 #include <locale.h>
+#ifdef ENABLE_UTF8
 #include <langinfo.h>
+#endif
 #include <termios.h>
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
@@ -502,6 +504,9 @@ openfilestruct *make_new_opennode(void)
     newnode->filebot = NULL;
     newnode->edittop = NULL;
     newnode->current = NULL;
+#ifndef NANO_TINY
+    newnode->last_action = OTHER;
+#endif
 
     return newnode;
 }
@@ -1642,6 +1647,9 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 	set_modified();
 
 #ifndef NANO_TINY
+	update_undo(ADD);
+
+
 	/* Note that current_x has not yet been incremented. */
 	if (openfile->mark_set && openfile->current ==
 		openfile->mark_begin && openfile->current_x <
@@ -2224,7 +2232,7 @@ int main(int argc, char **argv)
 		icol == 1)
 		parse_line_column(&argv[i][1], &iline, &icol);
 	    else {
-		open_buffer(argv[i]);
+		open_buffer(argv[i], FALSE);
 
 		if (iline > 1 || icol > 1) {
 		    do_gotolinecolumn(iline, icol, FALSE, FALSE, FALSE,
@@ -2241,14 +2249,14 @@ int main(int argc, char **argv)
      * buffer or a new buffer, depending on whether multibuffer mode is
      * enabled. */
     if (optind < argc)
-	open_buffer(argv[optind]);
+	open_buffer(argv[optind], FALSE);
 
     /* We didn't open any files if all the command line arguments were
      * invalid files like directories or if there were no command line
      * arguments given.  In this case, we have to load a blank buffer.
      * Also, we unset view mode to allow editing. */
     if (openfile == NULL) {
-	open_buffer("");
+	open_buffer("", FALSE);
 	UNSET(VIEW_MODE);
     }
 
