@@ -1,4 +1,4 @@
-/* $Id: nano.h 4237 2008-03-19 02:32:48Z astyanax $ */
+/* $Id: nano.h 4291 2008-08-03 04:48:05Z astyanax $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -169,6 +169,10 @@ typedef enum {
     CONTROL, META, FKEY, RAW
 }  function_type;
 
+typedef enum {
+    ADD, DEL, REPLACE, SPLIT, UNSPLIT, CUT, CUTTOEND, UNCUT, INSERT, OTHER
+} undo_type;
+
 /* Structure types. */
 typedef struct filestruct {
     char *data;
@@ -245,6 +249,40 @@ typedef struct syntaxtype {
 } syntaxtype;
 #endif /* ENABLE_COLOR */
 
+#ifndef NANO_TINY
+typedef struct undo {
+    ssize_t lineno;
+    undo_type type;
+	/* What type of undo was this */
+    int begin;
+	/* Where did this  action begin or end */
+    char *strdata;
+	/* String type data we will use for ccopying the affected line back */
+    int xflags;
+	/* Some flag data we need */
+
+    /* Cut specific stuff we need */
+    filestruct *cutbuffer;
+	/* Copy of the cutbuffer */
+    filestruct *cutbottom;
+	/* Copy of cutbottom */
+    bool mark_set;
+	/* was the marker set when we cut */
+    bool to_end;
+	/* was this a cut to end */
+    ssize_t mark_begin_lineno;
+	/* copy copy copy */
+    ssize_t mark_begin_x;
+	/* Another shadow variable */
+    ssize_t linescut;
+	/* How many lines we cut on a straight non-marked non-to-end cut */
+    struct undo *next;
+} undo;
+
+
+#endif /* NANO_TINY */
+
+
 typedef struct openfilestruct {
     char *filename;
 	/* The current file's name. */
@@ -278,6 +316,11 @@ typedef struct openfilestruct {
 	/* The current file's format. */
     struct stat *current_stat;
 	/* The current file's stat. */
+    undo *undotop;
+	/* Top of the undo list */
+    undo *current_undo;
+	/* The current (i.e. n ext) level of undo */
+    undo_type last_action;
 #endif
 #ifdef ENABLE_COLOR
     colortype *colorstrings;
@@ -318,25 +361,6 @@ typedef struct shortcut {
 	/* Next shortcut. */
 } shortcut;
 
-#ifndef NANO_TINY
-typedef struct toggle {
-   int val;
-	/* The sequence to toggle the key.  We should only need one. */
-   const char *desc;
-	/* The description of the toggle, e.g. "Cut to end"; we'll
-	 * append Enabled or Disabled to it. */
-#ifndef DISABLE_HELP
-   bool blank_after;
-	/* Whether there should be a blank line after the description of
-	 * the toggle. */
-#endif
-   long flag;
-	/* Which flag actually gets toggled. */
-   struct toggle *next;
-	/* Next toggle. */
-} toggle;
-#endif
-
 #ifdef ENABLE_NANORC
 typedef struct rcoption {
    const char *name;
@@ -667,6 +691,11 @@ typedef struct subnfunc {
 #define TOGGLE_BACKWARDS_KEY		NANO_META_B
 #define TOGGLE_DOS_KEY			NANO_META_D
 #define TOGGLE_MAC_KEY			NANO_META_M
+
+/* Extra bits for the undo function */
+#define UNDO_DEL_DEL		(1<<0)
+#define UNDO_DEL_BACKSPACE	(1<<1)
+
 #endif /* !NANO_TINY */
 
 #define VIEW TRUE
