@@ -1,4 +1,4 @@
-/* $Id: proto.h 4264 2008-06-03 08:09:05Z astyanax $ */
+/* $Id: proto.h 4291 2008-08-03 04:48:05Z astyanax $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -47,6 +47,7 @@ extern WINDOW *bottomwin;
 extern int editwinrows;
 
 extern filestruct *cutbuffer;
+extern filestruct *cutbottom;
 #ifndef DISABLE_JUSTIFY
 extern filestruct *jusbuffer;
 #endif
@@ -60,6 +61,7 @@ extern char *matchbrackets;
 #if !defined(NANO_TINY) && defined(ENABLE_NANORC)
 extern char *whitespace;
 extern int whitespace_len[2];
+extern undo_type last_action;
 #endif
 
 #ifndef DISABLE_JUSTIFY
@@ -100,9 +102,6 @@ extern char *syntaxstr;
 
 extern const shortcut *currshortcut;
 extern int currmenu;
-#ifndef NANO_TINY
-extern toggle *toggles;
-#endif
 
 #ifndef NANO_TINY
 extern filestruct *search_history;
@@ -240,7 +239,7 @@ void cut_to_eof(void);
 #endif
 void do_cut_text(
 #ifndef NANO_TINY
-	bool copy_text, bool cut_till_end
+	bool copy_text, bool cut_till_end, bool undoing
 #else
 	void
 #endif
@@ -256,7 +255,7 @@ void do_uncut_text(void);
 void make_new_buffer(void);
 void initialize_buffer(void);
 void initialize_buffer_text(void);
-void open_buffer(const char *filename);
+void open_buffer(const char *filename, bool undoable);
 #ifndef DISABLE_SPELLER
 void replace_buffer(const char *filename);
 #endif
@@ -269,7 +268,7 @@ bool close_buffer(void);
 #endif
 filestruct *read_line(char *buf, filestruct *prevnode, bool
 	*first_line_ins, size_t buf_len);
-void read_file(FILE *f, const char *filename);
+void read_file(FILE *f, const char *filename, bool undoable);
 int open_file(const char *filename, bool newfie, FILE **f);
 char *get_next_filename(const char *name, const char *suffix);
 void do_insertfile(
@@ -619,6 +618,8 @@ void do_tab(void);
 void do_indent(ssize_t cols);
 void do_indent_void(void);
 void do_unindent(void);
+void do_undo(void);
+void do_redo(void);
 #endif
 void do_enter(void);
 #ifndef NANO_TINY
@@ -706,8 +707,11 @@ void new_magicline(void);
 void remove_magicline(void);
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
 	**bot, size_t *bot_x, bool *right_side_up);
+void add_undo(undo_type current_action);
+void update_undo(undo_type action);
 #endif
 size_t get_totsize(const filestruct *begin, const filestruct *end);
+filestruct *fsfromline(ssize_t lineno);
 #ifdef DEBUG
 void dump_filestruct(const filestruct *inptr);
 void dump_filestruct_reverse(void);
@@ -739,9 +743,6 @@ int get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
 const sc *get_shortcut(int menu, int *kbinput, bool
 	*meta_key, bool *func_key);
 const sc *first_sc_for(int menu, void *func);
-#ifndef NANO_TINY
-const toggle *get_toggle(int kbinput, bool meta_key);
-#endif
 void blank_line(WINDOW *win, int y, int x, int n);
 void blank_titlebar(void);
 void blank_topbar(void);
