@@ -1,4 +1,4 @@
-/* $Id: rcfile.c 4265 2008-06-25 03:06:11Z astyanax $ */
+/* $Id: rcfile.c 4281 2008-07-13 16:44:19Z astyanax $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -380,7 +380,7 @@ void parse_keybinding(char *ptr)
     for (i = 0; i < strlen(keycopy); i++)
 	keycopy[i] = toupper(keycopy[i]);
 
-    if (keycopy[0] != 'M' && keycopy[0] != '^' && keycopy[0] != 'F') {
+    if (keycopy[0] != 'M' && keycopy[0] != '^' && keycopy[0] != 'F' && keycopy[0] != 'K') {
 	rcfile_error(
 		N_("keybindings must begin with \"^\", \"M\", or \"F\"\n"));
 	return;
@@ -423,7 +423,7 @@ void parse_keybinding(char *ptr)
 
 #ifdef DEBUG
     fprintf(stderr, "newsc now address %d, menu func assigned = %d, menu = %d\n",
-	(int) newsc, (int) newsc->scfunc, menu);
+	&newsc, &newsc->scfunc, menu);
 #endif
 
 
