@@ -1,4 +1,4 @@
-/* $Id: text.c 4260 2008-05-31 23:09:40Z astyanax $ */
+/* $Id: text.c 4300 2008-08-09 10:08:33Z astyanax $ */ 
 /**************************************************************************
  *   text.c                                                               *
  *                                                                        *
@@ -70,6 +70,10 @@ void do_delete(void)
 	/* Do we have to call edit_refresh(), or can we get away with
 	 * just update_line()? */
 
+#ifndef NANO_TINY
+    update_undo(DEL);
+#endif
+
     assert(openfile->current != NULL && openfile->current->data != NULL && openfile->current_x <= strlen(openfile->current->data));
 
     openfile->placewewant = xplustabs();
@@ -359,6 +363,284 @@ void do_unindent(void)
 {
     do_indent(-tabsize);
 }
+
+/* Undo the last thing(s) we did */
+void do_undo(void)
+{
+    undo *u = openfile->current_undo;
+    filestruct *f = openfile->current, *t;
+    int len = 0;
+    char *undidmsg, *data;
+    filestruct *oldcutbuffer = cutbuffer, *oldcutbottom = cutbottom;
+
+    if (!u) {
+	statusbar(_("Nothing in undo buffer!"));
+	return;
+    }
+
+
+    if (u->lineno <= f->lineno)
+        for (; f->prev != NULL && f->lineno != u->lineno; f = f->prev)
+	    ;
+    else
+        for (; f->next != NULL && f->lineno != u->lineno; f = f->next)
+	    ;
+    if (f->lineno != u->lineno) {
+        statusbar(_("Couldnt match current undo line"));
+	return;
+    }
+#ifdef DEBUG
+    fprintf(stderr, "data we're about to undo = \"%s\"\n", f->data);
+    fprintf(stderr, "Undo running for type %d\n", u->type);
+#endif
+
+    openfile->current_x = u->begin;
+    switch(u->type) {
+    case ADD:
+	undidmsg = _("text add");
+	len = strlen(f->data) - strlen(u->strdata) + 1;
+        data = charalloc(len);
+        strncpy(data, f->data, u->begin);
+	strcpy(&data[u->begin], &f->data[u->begin + strlen(u->strdata)]);
+	free(f->data);
+	f->data = data;
+	break;
+    case DEL:
+	undidmsg = _("text delete");
+	len = strlen(f->data) + strlen(u->strdata) + 1;
+	data = charalloc(len);
+
+	strncpy(data, f->data, u->begin);
+	strcpy(&data[u->begin], u->strdata);
+	strcpy(&data[u->begin + strlen(u->strdata)], &f->data[u->begin]);
+	free(f->data);
+	f->data = data;
+	if (u->xflags == UNDO_DEL_BACKSPACE)
+	    openfile->current_x += strlen(u->strdata);
+	break;
+    case SPLIT:
+	undidmsg = _("line split");
+	free(f->data);
+	f->data = mallocstrcpy(NULL, u->strdata);
+	if (f->next != NULL) {
+	    filestruct *tmp = f->next;
+	    unlink_node(tmp);
+	    delete_node(tmp);
+	}
+	renumber(f);
+	break;
+    case UNSPLIT:
+	undidmsg = _("line join");
+	t = make_new_node(f);
+	t->data = mallocstrcpy(NULL, u->strdata);
+	data = mallocstrncpy(NULL, f->data, u->begin);
+	data[u->begin] = '\0';
+	free(f->data);
+	f->data = data;
+	splice_node(f, t, f->next);
+	renumber(f);
+	break;
+    case CUT:
+    case CUTTOEND:
+	undidmsg = _("text cut");
+	cutbuffer = copy_filestruct(u->cutbuffer);
+
+        /* Compute cutbottom for the uncut using out copy */
+	for (cutbottom = cutbuffer; cutbottom->next != NULL; cutbottom = cutbottom->next)
+	    ;
+
+	/* Get to where we need to uncut from */
+	if (u->mark_set && u->mark_begin_lineno < u->lineno)
+	    do_gotolinecolumn(u->mark_begin_lineno, u->mark_begin_x+1, FALSE, FALSE, FALSE, FALSE);
+	else
+	    do_gotolinecolumn(u->lineno, u->begin+1, FALSE, FALSE, FALSE, FALSE);
+
+	do_uncut_text();
+	free_filestruct(cutbuffer);
+	cutbuffer = NULL;
+	break;
+    case INSERT:
+	undidmsg = _("text insert");
+	cutbuffer = NULL;
+	cutbottom = NULL;
+	/* When we updated mark_begin_lineno in update_undo, it was effectively how many line
+	   were inserted due to being partitioned before read_file was called.  So we
+	   add its value here */
+	openfile->mark_begin = fsfromline(u->lineno + u->mark_begin_lineno - 1);
+	openfile->mark_begin_x = 0;
+	openfile->mark_set = TRUE;
+	do_gotolinecolumn(u->lineno, u->begin+1, FALSE, FALSE, FALSE, FALSE);
+	cut_marked();
+	u->cutbuffer = cutbuffer;
+	u->cutbottom = cutbottom;
+	cutbuffer = oldcutbuffer;
+	cutbottom = oldcutbottom;
+	openfile->mark_set = FALSE;
+	break;
+    case REPLACE:
+	undidmsg = _("text replace");
+	data = u->strdata;
+	u->strdata = f->data;
+	f->data = data;
+	break;
+    default:
+	undidmsg = _("wtf?");
+	break;
+
+    }
+    do_gotolinecolumn(u->lineno, u->begin, FALSE, FALSE, FALSE, TRUE);
+    statusbar(_("Undid action (%s)"), undidmsg);
+    openfile->current_undo = openfile->current_undo->next;
+    openfile->last_action = OTHER;
+}
+
+void do_redo(void)
+{
+    undo *u = openfile->undotop;
+    filestruct *f = openfile->current, *t;
+    filestruct *oldcutbuffer = cutbuffer, *oldcutbottom = cutbottom;
+    int len = 0, i;
+    char *undidmsg, *data;
+
+    for (; u != NULL && u->next != openfile->current_undo; u = u->next)
+	;
+    if (!u) {
+	statusbar(_("Nothing to re-do!"));
+	return;
+    }
+    if (u->next != openfile->current_undo) {
+	statusbar(_("Can't find previous undo to re-do, argh"));
+	return;
+    }
+
+    if (u->lineno <= f->lineno)
+        for (; f->prev != NULL && f->lineno != u->lineno; f = f->prev)
+	    ;
+    else
+        for (; f->next != NULL && f->lineno != u->lineno; f = f->next)
+	    ;
+    if (f->lineno != u->lineno) {
+        statusbar(_("Couldnt match current undo line"));
+	return;
+    }
+#ifdef DEBUG
+    fprintf(stderr, "data we're about to redo = \"%s\"\n", f->data);
+    fprintf(stderr, "Redo running for type %d\n", u->type);
+#endif
+
+    switch(u->type) {
+    case ADD:
+	undidmsg = _("text add");
+	len = strlen(f->data) + strlen(u->strdata) + 1;
+        data = charalloc(len);
+	strncpy(data, f->data, u->begin);
+	strcpy(&data[u->begin], u->strdata);
+	strcpy(&data[u->begin + strlen(u->strdata)], &f->data[u->begin]);
+	free(f->data);
+	f->data = data;
+	break;
+    case DEL:
+	undidmsg = _("text delete");
+	len = strlen(f->data) + strlen(u->strdata) + 1;
+	data = charalloc(len);
+        strncpy(data, f->data, u->begin);
+	strcpy(&data[u->begin], &f->data[u->begin + strlen(u->strdata)]);
+	free(f->data);
+	f->data = data;
+	break;
+    case SPLIT:
+	undidmsg = _("line split");
+	t = make_new_node(f);
+	t->data = mallocstrcpy(NULL, &u->strdata[u->begin]);
+	data = mallocstrncpy(NULL, f->data, u->begin);
+	data[u->begin] = '\0';
+	free(f->data);
+	f->data = data;
+	splice_node(f, t, f->next);
+	renumber(f);
+	break;
+    case UNSPLIT:
+	undidmsg = _("line join");
+	len = strlen(f->data) + strlen(u->strdata + 1);
+	data = charalloc(len);
+	strcpy(data, f->data);
+	strcat(data, u->strdata);
+	free(f->data);
+	f->data = data;
+	if (f->next != NULL) {
+	    filestruct *tmp = f->next;
+	    unlink_node(tmp);
+	    delete_node(tmp);
+	}
+	renumber(f);
+	break;
+    case CUT:
+    case CUTTOEND:
+	undidmsg = _("line cut");
+	do_gotolinecolumn(u->lineno, u->begin+1, FALSE, FALSE, FALSE, FALSE);
+	openfile->mark_set = u->mark_set;
+	if (cutbuffer)
+	    free(cutbuffer);
+	cutbuffer = NULL;
+
+	/* Move ahead the same # lines we had if a marked cut */
+	if (u->mark_set) {
+	    for (i = 1, t = openfile->fileage; i != u->mark_begin_lineno; i++)
+		t = t->next;
+	    openfile->mark_begin = t;
+	} else if (!u->to_end) {
+	    /* Here we have a regular old potentially multi-line ^K cut.  We'll
+		need to trick nano into thinking it's a marked cut to cut more
+		than one line again */
+#ifdef DEBUG
+	    fprintf(stderr, "Undoing multi-^K cut, u->linescut = %d\n", u->linescut);
+#endif
+	    for (i = 0, t = openfile->current; i < u->linescut; i++) {
+
+#ifdef DEBUG
+	    fprintf(stderr, "Advancing, lineno  = %d, data = \"%s\"\n", t->lineno, t->data);
+#endif
+		t = t->next;
+	    }
+	    openfile->mark_begin = t;
+	    openfile->mark_begin_x = 0;
+	    openfile->mark_set = TRUE;
+	}
+
+	openfile->mark_begin_x = u->mark_begin_x;
+	do_cut_text(FALSE, u->to_end, TRUE);
+	openfile->mark_set = FALSE;
+        openfile->mark_begin = NULL;
+        openfile->mark_begin_x = 0;
+	edit_refresh();
+	break;
+    case REPLACE:
+	undidmsg = _("text replace");
+	data = u->strdata;
+	u->strdata = f->data;
+	f->data = data;
+	break;
+     case INSERT:
+	undidmsg = _("text insert");
+	cutbuffer = u->cutbuffer;
+	cutbottom = u->cutbottom;
+	do_gotolinecolumn(u->lineno, u->begin+1, FALSE, FALSE, FALSE, FALSE);
+	do_uncut_text();
+	cutbuffer = oldcutbuffer;
+	cutbottom = oldcutbottom;
+	break;
+    default:
+	undidmsg = _("wtf?");
+	break;
+
+    }
+    do_gotolinecolumn(u->lineno, u->begin, FALSE, FALSE, FALSE, TRUE);
+    statusbar(_("Redid action (%s)"), undidmsg);
+
+    openfile->current_undo = u;
+    openfile->last_action = OTHER;
+
+}
 #endif /* !NANO_TINY */
 
 /* Someone hits Enter *gasp!* */
@@ -367,9 +649,12 @@ void do_enter(void)
     filestruct *newnode = make_new_node(openfile->current);
     size_t extra = 0;
 
-    assert(openfile->current != NULL && openfile->current->data != NULL);
+    assert(openfile->current != NULL && xopenfile->current->data != NULL);
 
 #ifndef NANO_TINY
+    update_undo(SPLIT);
+
+
     /* Do auto-indenting, like the neolithic Turbo Pascal editor. */
     if (ISSET(AUTOINDENT)) {
 	/* If we are breaking the line in the indentation, the new
@@ -495,7 +780,7 @@ bool execute_command(const char *command)
     if (f == NULL)
 	nperror("fdopen");
 
-    read_file(f, "stdin");
+    read_file(f, "stdin", TRUE);
 
     if (wait(NULL) == -1)
 	nperror("wait");
@@ -510,6 +795,223 @@ bool execute_command(const char *command)
 
     return TRUE;
 }
+
+/* Add a new undo struct to the top of the current pile */
+void add_undo(undo_type current_action)
+{
+    undo *u;
+    char *data;
+    openfilestruct *fs = openfile;
+
+    /* Ugh, if we were called while cutting not-to-end, non-marked and on the same lineno,
+       we need to  abort here */
+    u = fs->current_undo;
+    if (current_action == CUT && u && u->type == CUT 
+	&& !u->mark_set && u->lineno == fs->current->lineno)
+	return;
+
+    /* Blow away the old undo stack if we are starting from the middle */
+    while (fs->undotop != NULL && fs->undotop != fs->current_undo) {
+	undo *u2 = fs->undotop;
+	fs->undotop = fs->undotop->next;
+	if (u2->strdata != NULL)
+	    free(u2->strdata);
+	if (u2->cutbuffer)
+	    free_filestruct(u2->cutbuffer);
+	free(u2);
+    }
+
+    /* Allocate and initialize a new undo type */
+    u = nmalloc(sizeof(undo));
+    u->type = current_action;
+    u->lineno = fs->current->lineno;
+    u->begin = fs->current_x;
+    u->next = fs->undotop;
+    fs->undotop = u;
+    fs->current_undo = u;
+    u->strdata = NULL;
+    u->cutbuffer = NULL;
+    u->cutbottom  = NULL;
+    u->mark_begin_lineno = 0;
+    u->mark_begin_x = 0;
+    u->linescut = 0;
+    u->xflags = 0;
+
+    switch (u->type) {
+    /* We need to start copying data into the undo buffer or we wont be able
+       to restore it later */
+    case ADD:
+        data = charalloc(2);
+        data[0] = fs->current->data[fs->current_x];
+        data[1] = '\0';
+        u->strdata = data;
+	break;
+    case DEL:
+	if (u->begin != strlen(fs->current->data)) {
+            data = mallocstrncpy(NULL, &fs->current->data[u->begin], 2);
+            data[1] = '\0';
+            u->strdata = data;
+	    break;
+	}
+	/* Else purposely fall into unsplit code */
+	current_action = u->type = UNSPLIT;
+    case UNSPLIT:
+	if (fs->current->next) {
+	    data = mallocstrcpy(NULL, fs->current->next->data);
+	    u->strdata = data;
+	}
+	break;
+    case INSERT:
+    case SPLIT:
+    case REPLACE:
+	data = mallocstrcpy(NULL, fs->current->data);
+	u->strdata = data;
+	break;
+    case CUT:
+    case CUTTOEND:
+	u->mark_set = openfile->mark_set;
+	if (u->mark_set) {
+	    u->mark_begin_lineno = openfile->mark_begin->lineno;
+	    u->mark_begin_x = openfile->mark_begin_x;
+	}
+	u->to_end = (current_action == CUTTOEND);
+	break;
+    case UNCUT:
+	break;
+    case OTHER:
+	statusbar(_("OOPS. Tried to add unknown thing to undo struct, I'd save your work"));
+	break;
+    }
+
+#ifdef DEBUG
+    fprintf(stderr, "fs->current->data = \"%s\", current_x = %d, u->begin = %d, type = %d\n",
+			fs->current->data,  fs->current_x, u->begin, current_action);
+    fprintf(stderr, "left update_add...\n");
+#endif
+    fs->last_action = current_action;
+}
+
+/* Update an undo item, or determine whether a new one
+   is really needed and bounce the data to add_undo
+   instead.  The latter functionality just feels
+   gimmicky and may just be more hassle than
+   it's worth, so it should be axed if needed. */
+void update_undo(undo_type action)
+{
+    undo *u;
+    char *data;
+    int len = 0;
+    openfilestruct *fs = openfile;
+
+#ifdef DEBUG
+        fprintf(stderr, "action = %d, fs->last_action = %d,  openfile->current->lineno = %d",
+		action, fs->last_action, openfile->current->lineno);
+	if (fs->current_undo)
+	    fprintf(stderr, "fs->current_undo->lineno = %d\n",  fs->current_undo->lineno);
+	else
+	    fprintf(stderr, "\n");
+#endif
+
+    /* Change to an add if we're not using the same undo struct
+       that we should be using */
+    if (action != fs->last_action
+	|| (action != CUT && action != CUTTOEND && action != INSERT
+	    && openfile->current->lineno != fs->current_undo->lineno)) {
+        add_undo(action);
+	return;
+    }
+
+    assert(fs->undotop != NULL);
+    u = fs->undotop;
+
+    switch (u->type) {
+    case ADD:
+#ifdef DEBUG
+        fprintf(stderr, "fs->current->data = \"%s\", current_x = %d, u->begin = %d\n",
+			fs->current->data,  fs->current_x, u->begin);
+#endif
+        len = strlen(u->strdata) + 2;
+        data = nrealloc((void *) u->strdata, len * sizeof(char *));
+        data[len-2] = fs->current->data[fs->current_x];
+        data[len-1] = '\0';
+        u->strdata = (char *) data;
+#ifdef DEBUG
+	fprintf(stderr, "current undo data now \"%s\"\n", u->strdata);
+#endif
+	break;
+    case DEL:
+	len = strlen(u->strdata) + 2;
+	assert(len > 2);
+        if (fs->current_x == u->begin) {
+	    /* They're deleting */
+	    if (!u->xflags)
+		u->xflags = UNDO_DEL_DEL;
+	    else if (u->xflags != UNDO_DEL_DEL) {
+		add_undo(action);
+		return;
+	    }
+	    data = charalloc(len);
+	    strcpy(data, u->strdata);
+	    data[len-2] = fs->current->data[fs->current_x];;
+	    data[len-1] = '\0';
+	    free(u->strdata);
+	    u->strdata = data;
+	} else if (fs->current_x == u->begin - 1) {
+	    /* They're backspacing */
+	    if (!u->xflags)
+		u->xflags = UNDO_DEL_BACKSPACE;
+	    else if (u->xflags != UNDO_DEL_BACKSPACE) {
+		add_undo(action);
+		return;
+	    }
+	    data = charalloc(len);
+	    data[0] = fs->current->data[fs->current_x];
+	    strcpy(&data[1], u->strdata);
+	    free(u->strdata);
+	    u->strdata = data;
+	    u->begin--;
+	} else {
+	    /* They deleted something else on the line */
+	    add_undo(DEL);
+	    return;
+	}
+#ifdef DEBUG
+	fprintf(stderr, "current undo data now \"%s\"\nu->begin = %d\n", u->strdata, u->begin);
+#endif
+	break;
+    case CUT:
+    case CUTTOEND:
+    case UNCUT:
+	if (u->cutbuffer)
+	    free(u->cutbuffer);
+	u->cutbuffer = copy_filestruct(cutbuffer);
+	u->cutbottom = cutbottom;
+	u->linescut++;
+	break;
+    case REPLACE:
+	add_undo(action);
+	break;
+    case INSERT:
+	u->mark_begin_lineno = openfile->current->lineno;
+    case SPLIT:
+    case UNSPLIT:
+	/* These cases are handled by the earlier check for a new line and action */
+    case OTHER:
+	break;
+    }
+
+#ifdef DEBUG
+    fprintf(stderr, "Done in udpate_undo (type was %d)\n", action);
+#endif
+    if (fs->last_action != action) {
+#ifdef DEBUG
+	fprintf(stderr, "Starting add_undo for new action as it does not match last_action\n");
+#endif
+	add_undo(action);
+    }
+    fs->last_action = action;
+}
+
 #endif /* !NANO_TINY */
 
 #ifndef DISABLE_WRAPPING
@@ -800,6 +1302,15 @@ ssize_t break_line(const char *line, ssize_t goal
 	/* In fact, the whole line displays shorter than goal. */
 	return cur_loc;
 
+#ifndef DISABLE_HELP
+    if (newln && blank_loc <= 0) {
+       /* If blank was not found or was found only first character,
+        * force line break. */
+       cur_loc -= line_len;
+       return cur_loc;
+    }
+#endif
+
     if (blank_loc == -1) {
 	/* No blank was found that was short enough. */
 	bool found_blank = FALSE;
@@ -2464,3 +2975,4 @@ void do_verbatim_input(void)
 
     free(output);
 }
+
